@@ -15,23 +15,25 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Save Rendered Template Fields """
+"""Save Rendered Template Fields"""
+import os
+from typing import Optional
 
 import sqlalchemy_jsonfield
 from sqlalchemy import Column, String, and_, not_, tuple_
+from sqlalchemy.orm import Session
 
 from airflow.configuration import conf
 from airflow.models.base import ID_LEN, Base
+from airflow.models.taskinstance import TaskInstance
 from airflow.serialization.helpers import serialize_template_field
 from airflow.settings import json
-from airflow.utils.db import provide_session
+from airflow.utils.session import provide_session
 from airflow.utils.sqlalchemy import UtcDateTime
 
 
 class RenderedTaskInstanceFields(Base):
-    """
-    Save Rendered Template Fields
-    """
+    """Save Rendered Template Fields"""
 
     __tablename__ = "rendered_task_instance_fields"
 
@@ -39,8 +41,9 @@ class RenderedTaskInstanceFields(Base):
     task_id = Column(String(ID_LEN), primary_key=True)
     execution_date = Column(UtcDateTime, primary_key=True)
     rendered_fields = Column(sqlalchemy_jsonfield.JSONField(json=json), nullable=False)
+    k8s_pod_yaml = Column(sqlalchemy_jsonfield.JSONField(json=json), nullable=True)
 
-    def __init__(self, ti, render_templates=True):
+    def __init__(self, ti: TaskInstance, render_templates=True):
         self.dag_id = ti.dag_id
         self.task_id = ti.task_id
         self.task = ti.task
@@ -48,19 +51,29 @@ class RenderedTaskInstanceFields(Base):
         self.ti = ti
         if render_templates:
             ti.render_templates()
+        if os.environ.get("AIRFLOW_IS_K8S_EXECUTOR_POD", None):
+            self.k8s_pod_yaml = ti.render_k8s_pod_yaml()
         self.rendered_fields = {
-            field: serialize_template_field(
-                getattr(self.task, field)
-            ) for field in self.task.template_fields
+            field: serialize_template_field(getattr(self.task, field)) for field in self.task.template_fields
         }
 
+        self._redact()
+
     def __repr__(self):
-        return "<{self.__class__.__name__}: {self.dag_id}.{self.task_id} {self.execution_date}".format(
-            self=self)
+        return f"<{self.__class__.__name__}: {self.dag_id}.{self.task_id} {self.execution_date}"
+
+    def _redact(self):
+        from airflow.utils.log.secrets_masker import redact
+
+        if self.k8s_pod_yaml:
+            self.k8s_pod_yaml = redact(self.k8s_pod_yaml)
+
+        for field, rendered in self.rendered_fields.items():
+            self.rendered_fields[field] = redact(rendered, field)
 
     @classmethod
     @provide_session
-    def get_templated_fields(cls, ti, session=None):
+    def get_templated_fields(cls, ti: TaskInstance, session: Session = None) -> Optional[dict]:
         """
         Get templated field for a TaskInstance from the RenderedTaskInstanceFields
         table.
@@ -69,11 +82,13 @@ class RenderedTaskInstanceFields(Base):
         :param session: SqlAlchemy Session
         :return: Rendered Templated TI field
         """
-        result = session.query(cls.rendered_fields).filter(
-            cls.dag_id == ti.dag_id,
-            cls.task_id == ti.task_id,
-            cls.execution_date == ti.execution_date
-        ).one_or_none()
+        result = (
+            session.query(cls.rendered_fields)
+            .filter(
+                cls.dag_id == ti.dag_id, cls.task_id == ti.task_id, cls.execution_date == ti.execution_date
+            )
+            .one_or_none()
+        )
 
         if result:
             rendered_fields = result.rendered_fields
@@ -81,8 +96,28 @@ class RenderedTaskInstanceFields(Base):
         else:
             return None
 
+    @classmethod
     @provide_session
-    def write(self, session=None):
+    def get_k8s_pod_yaml(cls, ti: TaskInstance, session: Session = None) -> Optional[dict]:
+        """
+        Get rendered Kubernetes Pod Yaml for a TaskInstance from the RenderedTaskInstanceFields
+        table.
+
+        :param ti: Task Instance
+        :param session: SqlAlchemy Session
+        :return: Kubernetes Pod Yaml
+        """
+        result = (
+            session.query(cls.k8s_pod_yaml)
+            .filter(
+                cls.dag_id == ti.dag_id, cls.task_id == ti.task_id, cls.execution_date == ti.execution_date
+            )
+            .one_or_none()
+        )
+        return result.k8s_pod_yaml if result else None
+
+    @provide_session
+    def write(self, session: Session = None):
         """Write instance to database
 
         :param session: SqlAlchemy Session
@@ -92,9 +127,11 @@ class RenderedTaskInstanceFields(Base):
     @classmethod
     @provide_session
     def delete_old_records(
-        cls, task_id, dag_id,
+        cls,
+        task_id: str,
+        dag_id: str,
         num_to_keep=conf.getint("core", "max_num_rendered_ti_fields_per_task", fallback=0),
-        session=None
+        session: Session = None,
     ):
         """
         Keep only Last X (num_to_keep) number of records for a task by deleting others
@@ -107,22 +144,22 @@ class RenderedTaskInstanceFields(Base):
         if num_to_keep <= 0:
             return
 
-        tis_to_keep_query = session \
-            .query(cls.dag_id, cls.task_id, cls.execution_date) \
-            .filter(cls.dag_id == dag_id, cls.task_id == task_id) \
-            .order_by(cls.execution_date.desc()) \
+        tis_to_keep_query = (
+            session.query(cls.dag_id, cls.task_id, cls.execution_date)
+            .filter(cls.dag_id == dag_id, cls.task_id == task_id)
+            .order_by(cls.execution_date.desc())
             .limit(num_to_keep)
+        )
 
         if session.bind.dialect.name in ["postgresql", "sqlite"]:
             # Fetch Top X records given dag_id & task_id ordered by Execution Date
             subq1 = tis_to_keep_query.subquery('subq1')
 
-            session.query(cls) \
-                .filter(and_(
-                    cls.dag_id == dag_id,
-                    cls.task_id == task_id,
-                    tuple_(cls.dag_id, cls.task_id, cls.execution_date).notin_(subq1))) \
-                .delete(synchronize_session=False)
+            session.query(cls).filter(
+                cls.dag_id == dag_id,
+                cls.task_id == task_id,
+                tuple_(cls.dag_id, cls.task_id, cls.execution_date).notin_(subq1),
+            ).delete(synchronize_session=False)
         elif session.bind.dialect.name in ["mysql"]:
             # Fetch Top X records given dag_id & task_id ordered by Execution Date
             subq1 = tis_to_keep_query.subquery('subq1')
@@ -131,28 +168,26 @@ class RenderedTaskInstanceFields(Base):
             # Workaround for MySQL Limitation (https://stackoverflow.com/a/19344141/5691525)
             # Limitation: This version of MySQL does not yet support
             # LIMIT & IN/ALL/ANY/SOME subquery
-            subq2 = (
-                session
-                .query(subq1.c.dag_id, subq1.c.task_id, subq1.c.execution_date)
-                .subquery('subq2')
-            )
+            subq2 = session.query(subq1.c.dag_id, subq1.c.task_id, subq1.c.execution_date).subquery('subq2')
 
-            session.query(cls) \
-                .filter(and_(
-                    cls.dag_id == dag_id,
-                    cls.task_id == task_id,
-                    tuple_(cls.dag_id, cls.task_id, cls.execution_date).notin_(subq2))) \
-                .delete(synchronize_session=False)
+            session.query(cls).filter(
+                cls.dag_id == dag_id,
+                cls.task_id == task_id,
+                tuple_(cls.dag_id, cls.task_id, cls.execution_date).notin_(subq2),
+            ).delete(synchronize_session=False)
         else:
             # Fetch Top X records given dag_id & task_id ordered by Execution Date
             tis_to_keep = tis_to_keep_query.all()
 
-            filter_tis = [not_(and_(
-                cls.dag_id == ti.dag_id,
-                cls.task_id == ti.task_id,
-                cls.execution_date == ti.execution_date
-            )) for ti in tis_to_keep]
+            filter_tis = [
+                not_(
+                    and_(
+                        cls.dag_id == ti.dag_id,
+                        cls.task_id == ti.task_id,
+                        cls.execution_date == ti.execution_date,
+                    )
+                )
+                for ti in tis_to_keep
+            ]
 
-            session.query(cls) \
-                .filter(and_(*filter_tis)) \
-                .delete(synchronize_session=False)
+            session.query(cls).filter(and_(*filter_tis)).delete(synchronize_session=False)
