@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
@@ -17,9 +16,8 @@
 # specific language governing permissions and limitations
 # under the License.
 
-from airflow.sensors.base_sensor_operator import BaseSensorOperator
+from airflow.sensors.base import BaseSensorOperator
 from airflow.utils import timezone
-from airflow.utils.decorators import apply_defaults
 
 
 class TimeSensor(BaseSensorOperator):
@@ -30,11 +28,10 @@ class TimeSensor(BaseSensorOperator):
     :type target_time: datetime.time
     """
 
-    @apply_defaults
-    def __init__(self, target_time, *args, **kwargs):
-        super(TimeSensor, self).__init__(*args, **kwargs)
+    def __init__(self, *, target_time, **kwargs):
+        super().__init__(**kwargs)
         self.target_time = target_time
 
     def poke(self, context):
         self.log.info('Checking if the time (%s) has come', self.target_time)
-        return timezone.utcnow().time() > self.target_time
+        return timezone.make_naive(timezone.utcnow(), self.dag.timezone).time() > self.target_time
